/*
 * SPDX-FileCopyrightText: Copyright (c) 2003-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 * SPDX-License-Identifier: MIT
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the Software),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#ifndef __ls10_dev_nxbar_tcp_global_ip_h__
#define __ls10_dev_nxbar_tcp_global_ip_h__
/* This file is autogenerated.  Do not edit */
#define NV_NXBAR_TCP            0x0001FFFF:0x0000000 /* RW--D */
#define NV_NXBAR_TCP_ERROR_STATUS                                                                        0x00000090     /* R--4R */
#define NV_NXBAR_TCP_ERROR_STATUS_TILE0                                                                  0:0            /* R--VF */
#define NV_NXBAR_TCP_ERROR_STATUS_TILE0_DEFAULT                                                          0x00000000     /* R---V */
#define NV_NXBAR_TCP_ERROR_STATUS_TILE1                                                                  1:1            /* R--VF */
#define NV_NXBAR_TCP_ERROR_STATUS_TILE1_DEFAULT                                                          0x00000000     /* R---V */
#define NV_NXBAR_TCP_ERROR_STATUS_TILE2                                                                  2:2            /* R--VF */
#define NV_NXBAR_TCP_ERROR_STATUS_TILE2_DEFAULT                                                          0x00000000     /* R---V */
#define NV_NXBAR_TCP_ERROR_STATUS_TILE3                                                                  3:3            /* R--VF */
#define NV_NXBAR_TCP_ERROR_STATUS_TILE3_DEFAULT                                                          0x00000000     /* R---V */
#define NV_NXBAR_TCP_ERROR_STATUS_TILE4                                                                  4:4            /* R--VF */
#define NV_NXBAR_TCP_ERROR_STATUS_TILE4_DEFAULT                                                          0x00000000     /* R---V */
#define NV_NXBAR_TCP_ERROR_STATUS_TILE5                                                                  5:5            /* R--VF */
#define NV_NXBAR_TCP_ERROR_STATUS_TILE5_DEFAULT                                                          0x00000000     /* R---V */
#define NV_NXBAR_TCP_ERROR_STATUS_TILE6                                                                  6:6            /* R--VF */
#define NV_NXBAR_TCP_ERROR_STATUS_TILE6_DEFAULT                                                          0x00000000     /* R---V */
#define NV_NXBAR_TCP_ERROR_STATUS_TILE7                                                                  7:7            /* R--VF */
#define NV_NXBAR_TCP_ERROR_STATUS_TILE7_DEFAULT                                                          0x00000000     /* R---V */
#define NV_NXBAR_TCP_ERROR_STATUS_TILE8                                                                  8:8            /* R--VF */
#define NV_NXBAR_TCP_ERROR_STATUS_TILE8_DEFAULT                                                          0x00000000     /* R---V */
#define NV_NXBAR_TCP_ERROR_STATUS_TILE9                                                                  9:9            /* R--VF */
#define NV_NXBAR_TCP_ERROR_STATUS_TILE9_DEFAULT                                                          0x00000000     /* R---V */
#define NV_NXBAR_TCP_ERROR_STATUS_TILE10                                                                 10:10          /* R--VF */
#define NV_NXBAR_TCP_ERROR_STATUS_TILE10_DEFAULT                                                         0x00000000     /* R---V */
#define NV_NXBAR_TCP_ERROR_STATUS_TILE11                                                                 11:11          /* R--VF */
#define NV_NXBAR_TCP_ERROR_STATUS_TILE11_DEFAULT                                                         0x00000000     /* R---V */
#define NV_NXBAR_TCP_ERROR_STATUS_TILEOUT0                                                               16:16          /* R--VF */
#define NV_NXBAR_TCP_ERROR_STATUS_TILEOUT0_DEFAULT                                                       0x00000000     /* R---V */
#define NV_NXBAR_TCP_ERROR_STATUS_TILEOUT1                                                               17:17          /* R--VF */
#define NV_NXBAR_TCP_ERROR_STATUS_TILEOUT1_DEFAULT                                                       0x00000000     /* R---V */
#define NV_NXBAR_TCP_ERROR_STATUS_TILEOUT2                                                               18:18          /* R--VF */
#define NV_NXBAR_TCP_ERROR_STATUS_TILEOUT2_DEFAULT                                                       0x00000000     /* R---V */
#define NV_NXBAR_TCP_ERROR_STATUS_TILEOUT3                                                               19:19          /* R--VF */
#define NV_NXBAR_TCP_ERROR_STATUS_TILEOUT3_DEFAULT                                                       0x00000000     /* R---V */
#define NV_NXBAR_TCP_ERROR_STATUS_TILEOUT4                                                               20:20          /* R--VF */
#define NV_NXBAR_TCP_ERROR_STATUS_TILEOUT4_DEFAULT                                                       0x00000000     /* R---V */
#define NV_NXBAR_TCP_ERROR_STATUS_TILEOUT5                                                               21:21          /* R--VF */
#define NV_NXBAR_TCP_ERROR_STATUS_TILEOUT5_DEFAULT                                                       0x00000000     /* R---V */
#define NV_NXBAR_TCP_ERROR_STATUS_TILEOUT6                                                               22:22          /* R--VF */
#define NV_NXBAR_TCP_ERROR_STATUS_TILEOUT6_DEFAULT                                                       0x00000000     /* R---V */
#define NV_NXBAR_TCP_ERROR_STATUS_TILEOUT7                                                               23:23          /* R--VF */
#define NV_NXBAR_TCP_ERROR_STATUS_TILEOUT7_DEFAULT                                                       0x00000000     /* R---V */
#define NV_NXBAR_TCP_ERROR_STATUS_TILEOUT8                                                               24:24          /* R--VF */
#define NV_NXBAR_TCP_ERROR_STATUS_TILEOUT8_DEFAULT                                                       0x00000000     /* R---V */
#define NV_NXBAR_TCP_ERROR_STATUS_TILEOUT9                                                               25:25          /* R--VF */
#define NV_NXBAR_TCP_ERROR_STATUS_TILEOUT9_DEFAULT                                                       0x00000000     /* R---V */
#define NV_NXBAR_TCP_ERROR_STATUS_TILEOUT10                                                              26:26          /* R--VF */
#define NV_NXBAR_TCP_ERROR_STATUS_TILEOUT10_DEFAULT                                                      0x00000000     /* R---V */
#endif // __ls10_dev_nxbar_tcp_global_ip_h__
