/*
 * SPDX-FileCopyrightText: Copyright (c) 2005-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 * SPDX-License-Identifier: MIT
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
 * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

#pragma once

//
// This file was generated with FINN, an NVIDIA coding tool.
// Source file:      ctrl/ctrl2080.finn
//




#include "ctrl/ctrlxxxx.h"
#include "ctrl2080/ctrl2080gpu.h"
#include "ctrl2080/ctrl2080fuse.h"
#include "ctrl2080/ctrl2080event.h"
#include "ctrl2080/ctrl2080tmr.h"
#include "ctrl2080/ctrl2080bios.h"
#include "ctrl2080/ctrl2080mc.h"
#include "ctrl2080/ctrl2080fifo.h"
#include "ctrl2080/ctrl2080fb.h"



#include "ctrl2080/ctrl2080spdm.h"
#include "ctrl2080/ctrl2080gr.h"
#include "ctrl2080/ctrl2080bus.h"
#include "ctrl2080/ctrl2080thermal.h"
#include "ctrl2080/ctrl2080fan.h"
#include "ctrl2080/ctrl2080i2c.h"
#include "ctrl2080/ctrl2080internal.h"
#include "ctrl2080/ctrl2080spi.h"
#include "ctrl2080/ctrl2080gpio.h"
#include "ctrl2080/ctrl2080clk.h"
#include "ctrl2080/ctrl2080perf.h"
#include "ctrl2080/ctrl2080perf_cf.h"


#include "ctrl2080/ctrl2080rc.h"
#include "ctrl2080/ctrl2080dma.h"
#include "ctrl2080/ctrl2080dmabuf.h"
#include "ctrl2080/ctrl2080nvd.h"
#include "ctrl2080/ctrl2080boardobj.h"
#include "ctrl2080/ctrl2080pmgr.h"
#include "ctrl2080/ctrl2080power.h"
#include "ctrl2080/ctrl2080lpwr.h"
#include "ctrl2080/ctrl2080acr.h"
#include "ctrl2080/ctrl2080ce.h"
#include "ctrl2080/ctrl2080nvlink.h"
#include "ctrl2080/ctrl2080flcn.h"
#include "ctrl2080/ctrl2080volt.h"
#include "ctrl2080/ctrl2080ecc.h"
#include "ctrl2080/ctrl2080cipher.h"
#include "ctrl2080/ctrl2080fla.h"
#include "ctrl2080/ctrl2080gsp.h"
#include "ctrl2080/ctrl2080pmu.h"


#include "ctrl2080/ctrl2080grmgr.h"


#include "ctrl2080/ctrl2080vgpumgrinternal.h"
#include "ctrl2080/ctrl2080hshub.h"
/* include appropriate os-specific command header */


#include "ctrl2080/ctrl2080unix.h"


